@@ -1,5 +1,5 @@
 /* cvm/client_xfer_command.c - CVM client command transmission library
- * Copyright (C)2006  Bruce Guenter <bruce@untroubled.org>
+ * Copyright (C)2008  Bruce Guenter <bruce@untroubled.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -72,12 +72,12 @@ static int waitforit(void)
   pid_t tmp;
   while ((tmp = wait(&status)) != -1) {
     if (tmp == pid)
-      return WIFEXITED(status) ? WEXITSTATUS(status) : 1;
+      return WIFEXITED(status) ? WEXITSTATUS(status) : -CVME_IO;
   }
-  return 1;
+  return -CVME_IO;
 }
 
-static int write_buffer(int fd, unsigned char* buffer, unsigned buflen)
+static int write_buffer(int fd, const unsigned char* buffer, unsigned buflen)
 {
   size_t wr;
 
@@ -103,23 +103,27 @@ static unsigned read_buffer(int fd, unsigned char* buffer)
   return buflen;
 }
 
-unsigned cvm_xfer_command(const char* module,
-			  unsigned char buffer[CVM_BUFSIZE],
-			  unsigned* buflen)
+unsigned cvm_xfer_command_packets(const char* module,
+				  const struct cvm_packet* request,
+				  struct cvm_packet* response)
 {
   int pipes[2];
   int result;
 
   if (!pipefork(module, pipes)) return CVME_IO;
   
-  if (!write_buffer(pipes[0], buffer, *buflen) ||
+  if (!write_buffer(pipes[0], request->data, request->length) ||
       close(pipes[0]) == -1 ||
-      (*buflen = read_buffer(pipes[1], buffer)) == 0 ||
+      (response->length = read_buffer(pipes[1], response->data)) == 0 ||
       close(pipes[1]) == -1) {
     killit();
-    if ((result = waitforit()) != 0) return result;
+    if ((result = waitforit()) < 0)
+      return -result;
     return CVME_IO;
   }
 
-  return waitforit();
+  if ((result = waitforit()) < 0)
+    return -result;
+  response->data[0] = result;
+  return 0;
 }
