@@ -31,28 +31,30 @@
 
 /* UDP module invocation *****************************************************/
 static int udp_sendrecv(int sock, ipv4addr* ip, ipv4port port,
-			unsigned char* buffer, unsigned* buflen)
+			const struct cvm_packet* request,
+			struct cvm_packet* response)
 {
   int timeout;
   int try;
   iopoll_fd ifd;
-  unsigned len = *buflen;
 
   ifd.fd = sock;
   ifd.events = IOPOLL_READ;
   for (timeout = 2, try = 0; try < 4; timeout *= 2, ++try) {
-    if ((unsigned)socket_send4(sock, buffer, len, ip, port) != len)
+    if ((unsigned)socket_send4(sock, request->data, request->length,
+			       ip, port) != request->length)
       return 0;
     if (iopoll(&ifd, 1, timeout*1000) != 0)
-      return (*buflen = socket_recv4(sock, buffer, CVM_BUFSIZE, ip, &port)) !=
-	(unsigned)-1;
+      return (response->length = socket_recv4(sock, response->data,
+					      CVM_BUFSIZE, ip,
+					      &port)) != (unsigned)-1;
   }
   return 0;
 }
 
-unsigned cvm_xfer_udp(const char* hostport,
-		      unsigned char buffer[CVM_BUFSIZE],
-		      unsigned* buflen)
+unsigned cvm_xfer_udp_packets(const char* hostport,
+			      const struct cvm_packet* request,
+			      struct cvm_packet* response)
 {
   static char* hostname;
   char* portstr;
@@ -61,18 +63,18 @@ unsigned cvm_xfer_udp(const char* hostport,
   struct hostent* he;
   ipv4addr ip;
   
-  if ((portstr = strchr(hostport, ':')) == 0) return 1;
+  if ((portstr = strchr(hostport, ':')) == 0) return CVME_GENERAL;
   if (hostname) free(hostname);
   hostname = malloc(portstr-hostport+1);
   memcpy(hostname, hostport, portstr-hostport);
   hostname[portstr-hostport] = 0;
   port = strtoul(portstr+1, &portstr, 10);
-  if (*portstr != 0) return 1;
-  if ((he = gethostbyname(hostname)) == 0) return 1;
+  if (*portstr != 0) return CVME_GENERAL;
+  if ((he = gethostbyname(hostname)) == 0) return CVME_GENERAL;
   memcpy(&ip, he->h_addr_list[0], 4);
   
   if ((sock = socket_udp()) == -1) return CVME_IO;
-  if (!udp_sendrecv(sock, &ip, port, buffer, buflen)) {
+  if (!udp_sendrecv(sock, &ip, port, request, response)) {
     close(sock);
     return CVME_IO;
   }
