@@ -98,267 +98,6 @@ run_compare_test() {
   tests_count=$(($tests_count+1))
 }
 
-##### Test tests/command-prefix #####
-
-runtest() {
-make_pwfile
-$src/cvm-testclient cvm-command:$src/cvm-pwfile pwfuser '' testpass
-}
-run_compare_test tests/command-prefix  <<END_OF_TEST_RESULTS
-user name:        pwfuser
-user ID:          123
-group ID:         456
-real name:        Gecos
-directory:        /home/ftp
-shell:            /bin/false
-group name:       (null)
-system user name: (null)
-system directory: (null)
-domain:           (null)
-mailbox path:     (null)
-END_OF_TEST_RESULTS
-
-
-##### Test tests/vmlookup-upper-domain #####
-
-runtest() {
-CVM_LOOKUP_SECRET=secret \
-$src/cvm-testclient $tmp/cvm-vmailmgr-lookup virt Test.TLD
-}
-run_compare_test tests/vmlookup-upper-domain  <<END_OF_TEST_RESULTS
-user name:        virt
-user ID:          @UID@
-group ID:         @GID@
-real name:        (null)
-directory:        @TMPDIR@/virt
-shell:            (null)
-group name:       (null)
-system user name: user
-system directory: @TMPDIR@
-domain:           test.tld
-mailbox path:     @TMPDIR@/virt
-END_OF_TEST_RESULTS
-
-
-##### Test tests/qmail-lookup-assume #####
-
-runtest() {
-env CVM_LOOKUP_SECRET=secret \
-$src/cvm-testclient $tmp/cvm-qmail-lookup user nothingness
-
-echo
-
-env CVM_LOOKUP_SECRET=secret CVM_QMAIL_ASSUME_LOCAL=1 \
-$src/cvm-testclient $tmp/cvm-qmail-lookup user nothingness
-}
-run_compare_test tests/qmail-lookup-assume  <<END_OF_TEST_RESULTS
-cvm-testclient: Fatal: Authentication failed, error #100 (Credentials rejected)
-
-user name:        user
-user ID:          @UID@
-group ID:         @GID@
-real name:        (null)
-directory:        @TMPDIR@
-shell:            (null)
-group name:       (null)
-system user name: user
-system directory: @TMPDIR@
-domain:           nothingness
-mailbox path:     @TMPDIR@
-END_OF_TEST_RESULTS
-
-
-##### Test tests/command-no-prefix #####
-
-runtest() {
-make_pwfile
-$src/cvm-testclient $src/cvm-pwfile pwfuser '' testpass
-}
-run_compare_test tests/command-no-prefix  <<END_OF_TEST_RESULTS
-user name:        pwfuser
-user ID:          123
-group ID:         456
-real name:        Gecos
-directory:        /home/ftp
-shell:            /bin/false
-group name:       (null)
-system user name: (null)
-system directory: (null)
-domain:           (null)
-mailbox path:     (null)
-END_OF_TEST_RESULTS
-
-
-##### Test tests/qmail-lookup-nodomain #####
-
-runtest() {
-CVM_LOOKUP_SECRET=secret \
-$src/cvm-testclient $tmp/cvm-qmail-lookup addr ''
-}
-run_compare_test tests/qmail-lookup-nodomain  <<END_OF_TEST_RESULTS
-user name:        user
-user ID:          500
-group ID:         500
-real name:        (null)
-directory:        /home/bruce/dev/cvm/tests-tmp
-shell:            (null)
-group name:       (null)
-system user name: user
-system directory: /home/bruce/dev/cvm/tests-tmp
-domain:           noathost
-mailbox path:     /home/bruce/dev/cvm/tests-tmp
-END_OF_TEST_RESULTS
-
-
-##### Test tests/chain #####
-
-runtest() {
-CVM_CHAIN0=$src/cvm-pwfile
-CVM_CHAIN1=$src/cvm-vmailmgr
-export CVM_CHAIN0 CVM_CHAIN1
-
-$src/cvm-testclient $src/cvm-chain pwfuser test.tld testpass
-echo
-$src/cvm-testclient $src/cvm-chain virt test.tld pass
-echo
-$src/cvm-testclient $src/cvm-chain notexist test.tld pass
-
-unset CVM_CHAIN0 CVM_CHAIN1
-}
-run_compare_test tests/chain  <<END_OF_TEST_RESULTS
-user name:        pwfuser
-user ID:          123
-group ID:         456
-real name:        Gecos
-directory:        /home/ftp
-shell:            /bin/false
-group name:       (null)
-system user name: (null)
-system directory: (null)
-domain:           (null)
-mailbox path:     (null)
-
-user name:        virt
-user ID:          @UID@
-group ID:         @GID@
-real name:        (null)
-directory:        @TMPDIR@/virt
-shell:            (null)
-group name:       (null)
-system user name: user
-system directory: @TMPDIR@
-domain:           test.tld
-mailbox path:     @TMPDIR@/virt
-
-cvm-testclient: Fatal: Authentication failed, error #100 (Credentials rejected)
-END_OF_TEST_RESULTS
-
-
-##### Test tests/command-bad-username #####
-
-runtest() {
-make_pwfile
-$src/cvm-testclient $src/cvm-pwfile pxfuser '' testpass
-}
-run_compare_test tests/command-bad-username  <<END_OF_TEST_RESULTS
-cvm-testclient: Fatal: Authentication failed, error #100 (Credentials rejected)
-END_OF_TEST_RESULTS
-
-
-##### Test tests/split #####
-
-runtest() {
-local dom1="$1"
-local dom2="$2"
-$src/cvm-testclient $src/cvm-vmailmgr virt$dom1 "$dom2" pass
-}
-run_compare_test tests/split '' '' <<END_OF_TEST_RESULTS
-user name:        virt
-user ID:          @UID@
-group ID:         @GID@
-real name:        (null)
-directory:        @TMPDIR@/virt
-shell:            (null)
-group name:       (null)
-system user name: user
-system directory: @TMPDIR@
-domain:           noathost
-mailbox path:     @TMPDIR@/virt
-END_OF_TEST_RESULTS
-
-run_compare_test tests/split '' 'test.tld' <<END_OF_TEST_RESULTS
-user name:        virt
-user ID:          @UID@
-group ID:         @GID@
-real name:        (null)
-directory:        @TMPDIR@/virt
-shell:            (null)
-group name:       (null)
-system user name: user
-system directory: @TMPDIR@
-domain:           test.tld
-mailbox path:     @TMPDIR@/virt
-END_OF_TEST_RESULTS
-
-run_compare_test tests/split '' 'other.tld' <<END_OF_TEST_RESULTS
-cvm-testclient: Fatal: Authentication failed, error #100 (Credentials rejected)
-END_OF_TEST_RESULTS
-
-run_compare_test tests/split '@test.tld' '' <<END_OF_TEST_RESULTS
-user name:        virt
-user ID:          @UID@
-group ID:         @GID@
-real name:        (null)
-directory:        @TMPDIR@/virt
-shell:            (null)
-group name:       (null)
-system user name: user
-system directory: @TMPDIR@
-domain:           test.tld
-mailbox path:     @TMPDIR@/virt
-END_OF_TEST_RESULTS
-
-run_compare_test tests/split '@test.tld' 'test.tld' <<END_OF_TEST_RESULTS
-user name:        virt
-user ID:          @UID@
-group ID:         @GID@
-real name:        (null)
-directory:        @TMPDIR@/virt
-shell:            (null)
-group name:       (null)
-system user name: user
-system directory: @TMPDIR@
-domain:           test.tld
-mailbox path:     @TMPDIR@/virt
-END_OF_TEST_RESULTS
-
-run_compare_test tests/split '@test.tld' 'other.tld' <<END_OF_TEST_RESULTS
-user name:        virt
-user ID:          @UID@
-group ID:         @GID@
-real name:        (null)
-directory:        @TMPDIR@/virt
-shell:            (null)
-group name:       (null)
-system user name: user
-system directory: @TMPDIR@
-domain:           test.tld
-mailbox path:     @TMPDIR@/virt
-END_OF_TEST_RESULTS
-
-run_compare_test tests/split '@other.tld' '' <<END_OF_TEST_RESULTS
-cvm-testclient: Fatal: Authentication failed, error #100 (Credentials rejected)
-END_OF_TEST_RESULTS
-
-run_compare_test tests/split '@other.tld' 'test.tld' <<END_OF_TEST_RESULTS
-cvm-testclient: Fatal: Authentication failed, error #100 (Credentials rejected)
-END_OF_TEST_RESULTS
-
-run_compare_test tests/split '@other.tld' 'other.tld' <<END_OF_TEST_RESULTS
-cvm-testclient: Fatal: Authentication failed, error #100 (Credentials rejected)
-END_OF_TEST_RESULTS
-
-
 ##### Test tests/sasl-eof #####
 
 runtest() {
@@ -373,35 +112,47 @@ sasl-auth-test: Fatal: sasl_auth1 failed: 8
 END_OF_TEST_RESULTS
 
 
-##### Test tests/pwfile-crypt #####
+##### Test tests/command-bad-password #####
 
 runtest() {
 make_pwfile
-CVM_PWFILE_PWCMP=crypt export CVM_PWFILE_PWCMP
-$src/cvm-testclient $src/cvm-pwfile cryptuser '' testpass
+$src/cvm-testclient $src/cvm-pwfile pwfuser '' testpassx
 }
-run_compare_test tests/pwfile-crypt  <<END_OF_TEST_RESULTS
-user name:        cryptuser
-user ID:          234
-group ID:         567
-real name:        Crypt
-directory:        /home/crypt
-shell:            /bin/true
+run_compare_test tests/command-bad-password  <<END_OF_TEST_RESULTS
+cvm-testclient: Authentication failed, error #100 (Credentials rejected)
+END_OF_TEST_RESULTS
+
+
+##### Test tests/chain-module #####
+
+runtest() {
+CVM_CHAIN0=$src/cvm-pwfile
+CVM_CHAIN1=$src/cvm-vmailmgr
+export CVM_CHAIN0 CVM_CHAIN1
+
+$src/cvm-testclient $src/cvm-chain pwfuser test.tld testpass
+echo
+$src/cvm-testclient $src/cvm-chain virt test.tld pass
+echo
+$src/cvm-testclient $src/cvm-chain notexist test.tld pass
+echo
+$src/cvm-testclient $src/cvm-chain notexist other.dom pass
+
+unset CVM_CHAIN0 CVM_CHAIN1
+}
+run_compare_test tests/chain-module  <<END_OF_TEST_RESULTS
+user name:        pwfuser
+user ID:          123
+group ID:         456
+real name:        Gecos
+directory:        /home/ftp
+shell:            /bin/false
 group name:       (null)
 system user name: (null)
 system directory: (null)
 domain:           (null)
 mailbox path:     (null)
-END_OF_TEST_RESULTS
 
-
-##### Test tests/vmlookup-normal #####
-
-runtest() {
-CVM_LOOKUP_SECRET=secret \
-$src/cvm-testclient $tmp/cvm-vmailmgr-lookup virt test.tld
-}
-run_compare_test tests/vmlookup-normal  <<END_OF_TEST_RESULTS
 user name:        virt
 user ID:          @UID@
 group ID:         @GID@
@@ -413,6 +164,28 @@ system user name: user
 system directory: @TMPDIR@
 domain:           test.tld
 mailbox path:     @TMPDIR@/virt
+
+cvm-testclient: Authentication failed, error #100 (Credentials rejected)
+out of scope:     0
+
+cvm-testclient: Authentication failed, error #100 (Credentials rejected)
+out of scope:     1
+END_OF_TEST_RESULTS
+
+
+##### Test tests/sasl-login1 #####
+
+runtest() {
+sasltest LOGIN <<EOF
+cHdmdXNlcg==
+dGVzdHBhc3M=
+EOF
+}
+run_compare_test tests/sasl-login1  <<END_OF_TEST_RESULTS
+sasl-auth-test: AUTH LOGIN PLAIN
++ VXNlcm5hbWU6^M
++ UGFzc3dvcmQ6^M
+sasl-auth-test: SASL AUTH LOGIN username=pwfuser
 END_OF_TEST_RESULTS
 
 
@@ -420,6 +193,8 @@ END_OF_TEST_RESULTS
 
 runtest() {
 $src/cvm-testclient $src/cvm-vmailmgr virt test.tld pass
+echo
+$src/cvm-testclient $src/cvm-vmailmgr missing test.tld pass
 }
 run_compare_test tests/vmailmgr-normal  <<END_OF_TEST_RESULTS
 user name:        virt
@@ -433,66 +208,107 @@ system user name: user
 system directory: @TMPDIR@
 domain:           test.tld
 mailbox path:     @TMPDIR@/virt
+
+cvm-testclient: Authentication failed, error #100 (Credentials rejected)
+out of scope:     0
 END_OF_TEST_RESULTS
 
 
-##### Test tests/vmlookup-pass #####
+##### Test tests/sasl-nomech #####
+
+runtest() {
+sasltest BOGUS </dev/null
+}
+run_compare_test tests/sasl-nomech  <<END_OF_TEST_RESULTS
+sasl-auth-test: AUTH LOGIN PLAIN
+sasl-auth-test: SASL AUTH BOGUS failed
+sasl-auth-test: Fatal: sasl_auth1 failed: 2
+  504 Unrecognized authentication mechanism.
+END_OF_TEST_RESULTS
+
+
+##### Test tests/lookup #####
+
+runtest() {
+local secret="$1"
+
+make_pwfile
+cat <<EOF >$tmp/cvmlookup
+#!/bin/sh
+CVM_LOOKUP_SECRET=$secret
+export CVM_LOOKUP_SECRET
+exec $src/cvm-pwfile
+EOF
+chmod +x $tmp/cvmlookup
+
+$src/cvm-testclient $tmp/cvmlookup pwfuser ''
+env CVM_LOOKUP_SECRET='' \
+$src/cvm-testclient $tmp/cvmlookup pwfuser ''
+env CVM_LOOKUP_SECRET=$secret \
+$src/cvm-testclient $tmp/cvmlookup pwfuser ''
+
+rm -f $tmp/cvmlookup
+}
+run_compare_test tests/lookup '' <<END_OF_TEST_RESULTS
+user name:        pwfuser
+user ID:          123
+group ID:         456
+real name:        Gecos
+directory:        /home/ftp
+shell:            /bin/false
+group name:       (null)
+system user name: (null)
+system directory: (null)
+domain:           (null)
+mailbox path:     (null)
+user name:        pwfuser
+user ID:          123
+group ID:         456
+real name:        Gecos
+directory:        /home/ftp
+shell:            /bin/false
+group name:       (null)
+system user name: (null)
+system directory: (null)
+domain:           (null)
+mailbox path:     (null)
+user name:        pwfuser
+user ID:          123
+group ID:         456
+real name:        Gecos
+directory:        /home/ftp
+shell:            /bin/false
+group name:       (null)
+system user name: (null)
+system directory: (null)
+domain:           (null)
+mailbox path:     (null)
+END_OF_TEST_RESULTS
+
+run_compare_test tests/lookup 'secret' <<END_OF_TEST_RESULTS
+cvm-testclient: Authentication failed, error #7 (Credential missing in data from module)
+cvm-testclient: Authentication failed, error #7 (Credential missing in data from module)
+user name:        pwfuser
+user ID:          123
+group ID:         456
+real name:        Gecos
+directory:        /home/ftp
+shell:            /bin/false
+group name:       (null)
+system user name: (null)
+system directory: (null)
+domain:           (null)
+mailbox path:     (null)
+END_OF_TEST_RESULTS
+
+
+##### Test tests/vmlookup-upper-virt #####
 
 runtest() {
 CVM_LOOKUP_SECRET=secret \
-$src/cvm-testclient $tmp/cvm-vmailmgr-lookup virt test.tld
+$src/cvm-testclient $tmp/cvm-vmailmgr-lookup Virt test.tld
 }
-run_compare_test tests/vmlookup-pass  <<END_OF_TEST_RESULTS
-user name:        virt
-user ID:          @UID@
-group ID:         @GID@
-real name:        (null)
-directory:        @TMPDIR@/virt
-shell:            (null)
-group name:       (null)
-system user name: user
-system directory: @TMPDIR@
-domain:           test.tld
-mailbox path:     @TMPDIR@/virt
-END_OF_TEST_RESULTS
-
-
-##### Test tests/vmailmgr-upper-pass #####
-
-runtest() {
-$src/cvm-testclient $src/cvm-vmailmgr virt test.tld Pass
-}
-run_compare_test tests/vmailmgr-upper-pass  <<END_OF_TEST_RESULTS
-cvm-testclient: Fatal: Authentication failed, error #100 (Credentials rejected)
-END_OF_TEST_RESULTS
-
-
-##### Test tests/vmailmgr-upper-virt #####
-
-runtest() {
-$src/cvm-testclient $src/cvm-vmailmgr Virt test.tld pass
-}
-run_compare_test tests/vmailmgr-upper-virt  <<END_OF_TEST_RESULTS
-user name:        virt
-user ID:          @UID@
-group ID:         @GID@
-real name:        (null)
-directory:        @TMPDIR@/virt
-shell:            (null)
-group name:       (null)
-system user name: user
-system directory: @TMPDIR@
-domain:           test.tld
-mailbox path:     @TMPDIR@/virt
-END_OF_TEST_RESULTS
-
-
-##### Test tests/vmailmgr-upper-domain #####
-
-runtest() {
-$src/cvm-testclient $src/cvm-vmailmgr virt Test.TLD pass
-}
-run_compare_test tests/vmailmgr-upper-domain  <<END_OF_TEST_RESULTS
+run_compare_test tests/vmlookup-upper-virt  <<END_OF_TEST_RESULTS
 user name:        virt
 user ID:          @UID@
 group ID:         @GID@
@@ -545,11 +361,13 @@ mailbox path:     @TMPDIR@
 END_OF_TEST_RESULTS
 
 run_compare_test tests/qmail-lookup 'user-addr-x' 'local.dom' <<END_OF_TEST_RESULTS
-cvm-testclient: Fatal: Authentication failed, error #100 (Credentials rejected)
+cvm-testclient: Authentication failed, error #100 (Credentials rejected)
+out of scope:     0
 END_OF_TEST_RESULTS
 
 run_compare_test tests/qmail-lookup 'user-wild' 'local.dom' <<END_OF_TEST_RESULTS
-cvm-testclient: Fatal: Authentication failed, error #100 (Credentials rejected)
+cvm-testclient: Authentication failed, error #100 (Credentials rejected)
+out of scope:     0
 END_OF_TEST_RESULTS
 
 run_compare_test tests/qmail-lookup 'user-wild-x' 'local.dom' <<END_OF_TEST_RESULTS
@@ -595,11 +413,13 @@ mailbox path:     @TMPDIR@/alias
 END_OF_TEST_RESULTS
 
 run_compare_test tests/qmail-lookup 'alias-x' 'local.dom' <<END_OF_TEST_RESULTS
-cvm-testclient: Fatal: Authentication failed, error #100 (Credentials rejected)
+cvm-testclient: Authentication failed, error #100 (Credentials rejected)
+out of scope:     0
 END_OF_TEST_RESULTS
 
 run_compare_test tests/qmail-lookup 'awild' 'local.dom' <<END_OF_TEST_RESULTS
-cvm-testclient: Fatal: Authentication failed, error #100 (Credentials rejected)
+cvm-testclient: Authentication failed, error #100 (Credentials rejected)
+out of scope:     0
 END_OF_TEST_RESULTS
 
 run_compare_test tests/qmail-lookup 'awild-x' 'local.dom' <<END_OF_TEST_RESULTS
@@ -687,222 +507,82 @@ mailbox path:     @TMPDIR@/alias
 END_OF_TEST_RESULTS
 
 
-##### Test tests/vmlookup-upper-virt #####
+##### Test tests/pwfile-bad-password #####
 
 runtest() {
-CVM_LOOKUP_SECRET=secret \
-$src/cvm-testclient $tmp/cvm-vmailmgr-lookup Virt test.tld
+make_pwfile
+$src/cvm-testclient $src/cvm-pwfile pwfuser '' testpaxx
 }
-run_compare_test tests/vmlookup-upper-virt  <<END_OF_TEST_RESULTS
-user name:        virt
+run_compare_test tests/pwfile-bad-password  <<END_OF_TEST_RESULTS
+cvm-testclient: Authentication failed, error #100 (Credentials rejected)
+END_OF_TEST_RESULTS
+
+
+##### Test tests/qmail-lookup-assume #####
+
+runtest() {
+env CVM_LOOKUP_SECRET=secret \
+$src/cvm-testclient $tmp/cvm-qmail-lookup user nothingness
+
+echo
+
+env CVM_LOOKUP_SECRET=secret CVM_QMAIL_ASSUME_LOCAL=1 \
+$src/cvm-testclient $tmp/cvm-qmail-lookup user nothingness
+}
+run_compare_test tests/qmail-lookup-assume  <<END_OF_TEST_RESULTS
+cvm-testclient: Authentication failed, error #100 (Credentials rejected)
+out of scope:     1
+
+user name:        user
 user ID:          @UID@
 group ID:         @GID@
 real name:        (null)
-directory:        @TMPDIR@/virt
+directory:        @TMPDIR@
 shell:            (null)
 group name:       (null)
 system user name: user
 system directory: @TMPDIR@
-domain:           test.tld
-mailbox path:     @TMPDIR@/virt
+domain:           nothingness
+mailbox path:     @TMPDIR@
 END_OF_TEST_RESULTS
 
 
-##### Test tests/lookup #####
+##### Test tests/sql-substitution #####
 
 runtest() {
-local secret="$1"
-
-make_pwfile
-cat <<EOF >$tmp/cvmlookup
-#!/bin/sh
-CVM_LOOKUP_SECRET=$secret
-export CVM_LOOKUP_SECRET
-exec $src/cvm-pwfile
-EOF
-chmod +x $tmp/cvmlookup
-
-$src/cvm-testclient $tmp/cvmlookup pwfuser ''
-env CVM_LOOKUP_SECRET='' \
-$src/cvm-testclient $tmp/cvmlookup pwfuser ''
-env CVM_LOOKUP_SECRET=$secret \
-$src/cvm-testclient $tmp/cvmlookup pwfuser ''
-
-rm -f $tmp/cvmlookup
+sqt() { env - var1=1 var_2=2 $src/sql-query-test "$@"; }
+sqt 'one two' act dom
+sqt '$account $domain one two' act dom
+sqt 'one $account two' act dom
+sqt 'one two $account' act dom
+sqt '$var1 one two' act dom
+sqt 'one $var1 two' act dom
+sqt 'one two $var1' act dom
+sqt '$var_2 one two' act dom
+sqt 'one $var_2 two' act dom
+sqt 'one two $var_2' act dom
+sqt '$var1one two' act dom
+sqt 'one $var1two' act dom
+sqt '${var1} one two' act dom
+sqt 'one ${var1} two' act dom
+sqt 'one two ${var1}' act dom
 }
-run_compare_test tests/lookup '' <<END_OF_TEST_RESULTS
-user name:        pwfuser
-user ID:          123
-group ID:         456
-real name:        Gecos
-directory:        /home/ftp
-shell:            /bin/false
-group name:       (null)
-system user name: (null)
-system directory: (null)
-domain:           (null)
-mailbox path:     (null)
-user name:        pwfuser
-user ID:          123
-group ID:         456
-real name:        Gecos
-directory:        /home/ftp
-shell:            /bin/false
-group name:       (null)
-system user name: (null)
-system directory: (null)
-domain:           (null)
-mailbox path:     (null)
-user name:        pwfuser
-user ID:          123
-group ID:         456
-real name:        Gecos
-directory:        /home/ftp
-shell:            /bin/false
-group name:       (null)
-system user name: (null)
-system directory: (null)
-domain:           (null)
-mailbox path:     (null)
-END_OF_TEST_RESULTS
-
-run_compare_test tests/lookup 'secret' <<END_OF_TEST_RESULTS
-cvm-testclient: Fatal: Authentication failed, error #7 (Credential missing in data from module)
-cvm-testclient: Fatal: Authentication failed, error #7 (Credential missing in data from module)
-user name:        pwfuser
-user ID:          123
-group ID:         456
-real name:        Gecos
-directory:        /home/ftp
-shell:            /bin/false
-group name:       (null)
-system user name: (null)
-system directory: (null)
-domain:           (null)
-mailbox path:     (null)
-END_OF_TEST_RESULTS
-
-
-##### Test tests/vmlookup-badpass #####
-
-runtest() {
-$src/cvm-testclient $src/cvm-vmailmgr virt test.tld 'xpass'
-}
-run_compare_test tests/vmlookup-badpass  <<END_OF_TEST_RESULTS
-cvm-testclient: Fatal: Authentication failed, error #100 (Credentials rejected)
-END_OF_TEST_RESULTS
-
-
-##### Test tests/pwfile #####
-
-runtest() {
-make_pwfile
-$src/cvm-testclient $src/cvm-pwfile pwfuser '' testpass
-}
-run_compare_test tests/pwfile  <<END_OF_TEST_RESULTS
-user name:        pwfuser
-user ID:          123
-group ID:         456
-real name:        Gecos
-directory:        /home/ftp
-shell:            /bin/false
-group name:       (null)
-system user name: (null)
-system directory: (null)
-domain:           (null)
-mailbox path:     (null)
-END_OF_TEST_RESULTS
-
-
-##### Test tests/sasl-login1 #####
-
-runtest() {
-sasltest LOGIN <<EOF
-cHdmdXNlcg==
-dGVzdHBhc3M=
-EOF
-}
-run_compare_test tests/sasl-login1  <<END_OF_TEST_RESULTS
-sasl-auth-test: AUTH LOGIN PLAIN
-+ VXNlcm5hbWU6^M
-+ UGFzc3dvcmQ6^M
-sasl-auth-test: SASL AUTH LOGIN username=pwfuser
-END_OF_TEST_RESULTS
-
-
-##### Test tests/sasl-login2 #####
-
-runtest() {
-sasltest LOGIN\ cHdmdXNlcg== <<EOF
-dGVzdHBhc3M=
-EOF
-}
-run_compare_test tests/sasl-login2  <<END_OF_TEST_RESULTS
-sasl-auth-test: AUTH LOGIN PLAIN
-+ UGFzc3dvcmQ6^M
-sasl-auth-test: SASL AUTH LOGIN username=pwfuser
-END_OF_TEST_RESULTS
-
-
-##### Test tests/sasl-nomech #####
-
-runtest() {
-sasltest BOGUS </dev/null
-}
-run_compare_test tests/sasl-nomech  <<END_OF_TEST_RESULTS
-sasl-auth-test: AUTH LOGIN PLAIN
-sasl-auth-test: SASL AUTH BOGUS failed
-sasl-auth-test: Fatal: sasl_auth1 failed: 2
-  504 Unrecognized authentication mechanism.
-END_OF_TEST_RESULTS
-
-
-##### Test tests/sasl-plain1 #####
-
-runtest() {
-sasltest PLAIN <<EOF
-cHdmdXNlcgBwd2Z1c2VyAHRlc3RwYXNz
-EOF
-}
-run_compare_test tests/sasl-plain1  <<END_OF_TEST_RESULTS
-sasl-auth-test: AUTH LOGIN PLAIN
-+ ^M
-sasl-auth-test: SASL AUTH PLAIN username=pwfuser
-END_OF_TEST_RESULTS
-
-
-##### Test tests/sasl-plain2 #####
-
-runtest() {
-sasltest PLAIN\ cHdmdXNlcgBwd2Z1c2VyAHRlc3RwYXNz </dev/null
-}
-run_compare_test tests/sasl-plain2  <<END_OF_TEST_RESULTS
-sasl-auth-test: AUTH LOGIN PLAIN
-sasl-auth-test: SASL AUTH PLAIN username=pwfuser
-END_OF_TEST_RESULTS
-
-
-##### Test tests/command-bad-password #####
-
-runtest() {
-make_pwfile
-$src/cvm-testclient $src/cvm-pwfile pwfuser '' testpassx
-}
-run_compare_test tests/command-bad-password  <<END_OF_TEST_RESULTS
-cvm-testclient: Fatal: Authentication failed, error #100 (Credentials rejected)
-END_OF_TEST_RESULTS
-
-
-##### Test tests/sasl-plain-fail #####
-
-runtest() {
-sasltest PLAIN\ cHdmdXNlcgBwd2Z1c2VyAHRlc3RwYXNZ </dev/null
-}
-run_compare_test tests/sasl-plain-fail  <<END_OF_TEST_RESULTS
-sasl-auth-test: AUTH LOGIN PLAIN
-sasl-auth-test: SASL AUTH PLAIN failed
-sasl-auth-test: Fatal: sasl_auth1 failed: 1
-  501 Authentication failed.
+run_compare_test tests/sql-substitution  <<END_OF_TEST_RESULTS
+one two
+'act' 'dom' one two
+one 'act' two
+one two 'act'
+'1' one two
+one '1' two
+one two '1'
+'2' one two
+one '2' two
+one two '2'
+ two
+one 
+'1' one two
+one '1' two
+one two '1'
 END_OF_TEST_RESULTS
 
 
@@ -979,42 +659,405 @@ mailbox path:     (null)
 END_OF_TEST_RESULTS
 
 
-##### Test tests/sql-substitution #####
+##### Test tests/vmailmgr-upper-pass #####
 
 runtest() {
-sqt() { env - var1=1 var_2=2 $src/sql-query-test "$@"; }
-sqt 'one two' act dom
-sqt '$account $domain one two' act dom
-sqt 'one $account two' act dom
-sqt 'one two $account' act dom
-sqt '$var1 one two' act dom
-sqt 'one $var1 two' act dom
-sqt 'one two $var1' act dom
-sqt '$var_2 one two' act dom
-sqt 'one $var_2 two' act dom
-sqt 'one two $var_2' act dom
-sqt '$var1one two' act dom
-sqt 'one $var1two' act dom
-sqt '${var1} one two' act dom
-sqt 'one ${var1} two' act dom
-sqt 'one two ${var1}' act dom
+$src/cvm-testclient $src/cvm-vmailmgr virt test.tld Pass
 }
-run_compare_test tests/sql-substitution  <<END_OF_TEST_RESULTS
-one two
-'act' 'dom' one two
-one 'act' two
-one two 'act'
-'1' one two
-one '1' two
-one two '1'
-'2' one two
-one '2' two
-one two '2'
- two
-one 
-'1' one two
-one '1' two
-one two '1'
+run_compare_test tests/vmailmgr-upper-pass  <<END_OF_TEST_RESULTS
+cvm-testclient: Authentication failed, error #100 (Credentials rejected)
+out of scope:     0
+END_OF_TEST_RESULTS
+
+
+##### Test tests/split #####
+
+runtest() {
+local dom1="$1"
+local dom2="$2"
+$src/cvm-testclient $src/cvm-vmailmgr virt$dom1 "$dom2" pass
+}
+run_compare_test tests/split '' '' <<END_OF_TEST_RESULTS
+user name:        virt
+user ID:          @UID@
+group ID:         @GID@
+real name:        (null)
+directory:        @TMPDIR@/virt
+shell:            (null)
+group name:       (null)
+system user name: user
+system directory: @TMPDIR@
+domain:           noathost
+mailbox path:     @TMPDIR@/virt
+END_OF_TEST_RESULTS
+
+run_compare_test tests/split '' 'test.tld' <<END_OF_TEST_RESULTS
+user name:        virt
+user ID:          @UID@
+group ID:         @GID@
+real name:        (null)
+directory:        @TMPDIR@/virt
+shell:            (null)
+group name:       (null)
+system user name: user
+system directory: @TMPDIR@
+domain:           test.tld
+mailbox path:     @TMPDIR@/virt
+END_OF_TEST_RESULTS
+
+run_compare_test tests/split '' 'other.tld' <<END_OF_TEST_RESULTS
+cvm-testclient: Authentication failed, error #100 (Credentials rejected)
+out of scope:     1
+END_OF_TEST_RESULTS
+
+run_compare_test tests/split '@test.tld' '' <<END_OF_TEST_RESULTS
+user name:        virt
+user ID:          @UID@
+group ID:         @GID@
+real name:        (null)
+directory:        @TMPDIR@/virt
+shell:            (null)
+group name:       (null)
+system user name: user
+system directory: @TMPDIR@
+domain:           test.tld
+mailbox path:     @TMPDIR@/virt
+END_OF_TEST_RESULTS
+
+run_compare_test tests/split '@test.tld' 'test.tld' <<END_OF_TEST_RESULTS
+user name:        virt
+user ID:          @UID@
+group ID:         @GID@
+real name:        (null)
+directory:        @TMPDIR@/virt
+shell:            (null)
+group name:       (null)
+system user name: user
+system directory: @TMPDIR@
+domain:           test.tld
+mailbox path:     @TMPDIR@/virt
+END_OF_TEST_RESULTS
+
+run_compare_test tests/split '@test.tld' 'other.tld' <<END_OF_TEST_RESULTS
+user name:        virt
+user ID:          @UID@
+group ID:         @GID@
+real name:        (null)
+directory:        @TMPDIR@/virt
+shell:            (null)
+group name:       (null)
+system user name: user
+system directory: @TMPDIR@
+domain:           test.tld
+mailbox path:     @TMPDIR@/virt
+END_OF_TEST_RESULTS
+
+run_compare_test tests/split '@other.tld' '' <<END_OF_TEST_RESULTS
+cvm-testclient: Authentication failed, error #100 (Credentials rejected)
+out of scope:     1
+END_OF_TEST_RESULTS
+
+run_compare_test tests/split '@other.tld' 'test.tld' <<END_OF_TEST_RESULTS
+cvm-testclient: Authentication failed, error #100 (Credentials rejected)
+out of scope:     1
+END_OF_TEST_RESULTS
+
+run_compare_test tests/split '@other.tld' 'other.tld' <<END_OF_TEST_RESULTS
+cvm-testclient: Authentication failed, error #100 (Credentials rejected)
+out of scope:     1
+END_OF_TEST_RESULTS
+
+
+##### Test tests/vmlookup-normal #####
+
+runtest() {
+CVM_LOOKUP_SECRET=secret \
+$src/cvm-testclient $tmp/cvm-vmailmgr-lookup virt test.tld
+}
+run_compare_test tests/vmlookup-normal  <<END_OF_TEST_RESULTS
+user name:        virt
+user ID:          @UID@
+group ID:         @GID@
+real name:        (null)
+directory:        @TMPDIR@/virt
+shell:            (null)
+group name:       (null)
+system user name: user
+system directory: @TMPDIR@
+domain:           test.tld
+mailbox path:     @TMPDIR@/virt
+END_OF_TEST_RESULTS
+
+
+##### Test tests/vmlookup-upper-domain #####
+
+runtest() {
+CVM_LOOKUP_SECRET=secret \
+$src/cvm-testclient $tmp/cvm-vmailmgr-lookup virt Test.TLD
+}
+run_compare_test tests/vmlookup-upper-domain  <<END_OF_TEST_RESULTS
+user name:        virt
+user ID:          @UID@
+group ID:         @GID@
+real name:        (null)
+directory:        @TMPDIR@/virt
+shell:            (null)
+group name:       (null)
+system user name: user
+system directory: @TMPDIR@
+domain:           test.tld
+mailbox path:     @TMPDIR@/virt
+END_OF_TEST_RESULTS
+
+
+##### Test tests/sasl-plain1 #####
+
+runtest() {
+sasltest PLAIN <<EOF
+cHdmdXNlcgBwd2Z1c2VyAHRlc3RwYXNz
+EOF
+}
+run_compare_test tests/sasl-plain1  <<END_OF_TEST_RESULTS
+sasl-auth-test: AUTH LOGIN PLAIN
++ ^M
+sasl-auth-test: SASL AUTH PLAIN username=pwfuser
+END_OF_TEST_RESULTS
+
+
+##### Test tests/vmlookup-pass #####
+
+runtest() {
+CVM_LOOKUP_SECRET=secret \
+$src/cvm-testclient $tmp/cvm-vmailmgr-lookup virt test.tld
+}
+run_compare_test tests/vmlookup-pass  <<END_OF_TEST_RESULTS
+user name:        virt
+user ID:          @UID@
+group ID:         @GID@
+real name:        (null)
+directory:        @TMPDIR@/virt
+shell:            (null)
+group name:       (null)
+system user name: user
+system directory: @TMPDIR@
+domain:           test.tld
+mailbox path:     @TMPDIR@/virt
+END_OF_TEST_RESULTS
+
+
+##### Test tests/command-bad-username #####
+
+runtest() {
+make_pwfile
+$src/cvm-testclient $src/cvm-pwfile pxfuser '' testpass
+}
+run_compare_test tests/command-bad-username  <<END_OF_TEST_RESULTS
+cvm-testclient: Authentication failed, error #100 (Credentials rejected)
+END_OF_TEST_RESULTS
+
+
+##### Test tests/command-no-prefix #####
+
+runtest() {
+make_pwfile
+$src/cvm-testclient $src/cvm-pwfile pwfuser '' testpass
+}
+run_compare_test tests/command-no-prefix  <<END_OF_TEST_RESULTS
+user name:        pwfuser
+user ID:          123
+group ID:         456
+real name:        Gecos
+directory:        /home/ftp
+shell:            /bin/false
+group name:       (null)
+system user name: (null)
+system directory: (null)
+domain:           (null)
+mailbox path:     (null)
+END_OF_TEST_RESULTS
+
+
+##### Test tests/sasl-plain-fail #####
+
+runtest() {
+sasltest PLAIN\ cHdmdXNlcgBwd2Z1c2VyAHRlc3RwYXNZ </dev/null
+}
+run_compare_test tests/sasl-plain-fail  <<END_OF_TEST_RESULTS
+sasl-auth-test: AUTH LOGIN PLAIN
+sasl-auth-test: SASL AUTH PLAIN failed
+sasl-auth-test: Fatal: sasl_auth1 failed: 1
+  501 Authentication failed.
+END_OF_TEST_RESULTS
+
+
+##### Test tests/qmail-lookup-nodomain #####
+
+runtest() {
+CVM_LOOKUP_SECRET=secret \
+$src/cvm-testclient $tmp/cvm-qmail-lookup addr ''
+}
+run_compare_test tests/qmail-lookup-nodomain  <<END_OF_TEST_RESULTS
+user name:        user
+user ID:          500
+group ID:         500
+real name:        (null)
+directory:        /home/bruce/dev/cvm/tests-tmp
+shell:            (null)
+group name:       (null)
+system user name: user
+system directory: /home/bruce/dev/cvm/tests-tmp
+domain:           noathost
+mailbox path:     /home/bruce/dev/cvm/tests-tmp
+END_OF_TEST_RESULTS
+
+
+##### Test tests/vmailmgr-upper-virt #####
+
+runtest() {
+$src/cvm-testclient $src/cvm-vmailmgr Virt test.tld pass
+}
+run_compare_test tests/vmailmgr-upper-virt  <<END_OF_TEST_RESULTS
+user name:        virt
+user ID:          @UID@
+group ID:         @GID@
+real name:        (null)
+directory:        @TMPDIR@/virt
+shell:            (null)
+group name:       (null)
+system user name: user
+system directory: @TMPDIR@
+domain:           test.tld
+mailbox path:     @TMPDIR@/virt
+END_OF_TEST_RESULTS
+
+
+##### Test tests/sasl-plain2 #####
+
+runtest() {
+sasltest PLAIN\ cHdmdXNlcgBwd2Z1c2VyAHRlc3RwYXNz </dev/null
+}
+run_compare_test tests/sasl-plain2  <<END_OF_TEST_RESULTS
+sasl-auth-test: AUTH LOGIN PLAIN
+sasl-auth-test: SASL AUTH PLAIN username=pwfuser
+END_OF_TEST_RESULTS
+
+
+##### Test tests/pwfile-crypt #####
+
+runtest() {
+make_pwfile
+CVM_PWFILE_PWCMP=crypt export CVM_PWFILE_PWCMP
+$src/cvm-testclient $src/cvm-pwfile cryptuser '' testpass
+}
+run_compare_test tests/pwfile-crypt  <<END_OF_TEST_RESULTS
+user name:        cryptuser
+user ID:          234
+group ID:         567
+real name:        Crypt
+directory:        /home/crypt
+shell:            /bin/true
+group name:       (null)
+system user name: (null)
+system directory: (null)
+domain:           (null)
+mailbox path:     (null)
+END_OF_TEST_RESULTS
+
+
+##### Test tests/command-case #####
+
+runtest() {
+make_pwfile
+$src/cvm-testclient $src/cvm-pwfile PWFUser '' testpass
+}
+run_compare_test tests/command-case  <<END_OF_TEST_RESULTS
+user name:        pwfuser
+user ID:          123
+group ID:         456
+real name:        Gecos
+directory:        /home/ftp
+shell:            /bin/false
+group name:       (null)
+system user name: (null)
+system directory: (null)
+domain:           (null)
+mailbox path:     (null)
+END_OF_TEST_RESULTS
+
+
+##### Test tests/command-prefix #####
+
+runtest() {
+make_pwfile
+$src/cvm-testclient cvm-command:$src/cvm-pwfile pwfuser '' testpass
+}
+run_compare_test tests/command-prefix  <<END_OF_TEST_RESULTS
+user name:        pwfuser
+user ID:          123
+group ID:         456
+real name:        Gecos
+directory:        /home/ftp
+shell:            /bin/false
+group name:       (null)
+system user name: (null)
+system directory: (null)
+domain:           (null)
+mailbox path:     (null)
+END_OF_TEST_RESULTS
+
+
+##### Test tests/vmlookup-badpass #####
+
+runtest() {
+$src/cvm-testclient $src/cvm-vmailmgr virt test.tld 'xpass'
+}
+run_compare_test tests/vmlookup-badpass  <<END_OF_TEST_RESULTS
+cvm-testclient: Authentication failed, error #100 (Credentials rejected)
+out of scope:     0
+END_OF_TEST_RESULTS
+
+
+##### Test tests/chain-client #####
+
+runtest() {
+$src/cvm-testclient $src/cvm-pwfile,$src/cvm-vmailmgr pwfuser test.tld testpass
+echo
+$src/cvm-testclient $src/cvm-pwfile,$src/cvm-vmailmgr virt test.tld pass
+echo
+$src/cvm-testclient $src/cvm-pwfile,$src/cvm-vmailmgr notexist test.tld pass
+echo
+$src/cvm-testclient $src/cvm-pwfile,$src/cvm-vmailmgr notexist other.dom pass
+}
+run_compare_test tests/chain-client  <<END_OF_TEST_RESULTS
+user name:        pwfuser
+user ID:          123
+group ID:         456
+real name:        Gecos
+directory:        /home/ftp
+shell:            /bin/false
+group name:       (null)
+system user name: (null)
+system directory: (null)
+domain:           (null)
+mailbox path:     (null)
+
+user name:        virt
+user ID:          @UID@
+group ID:         @GID@
+real name:        (null)
+directory:        @TMPDIR@/virt
+shell:            (null)
+group name:       (null)
+system user name: user
+system directory: @TMPDIR@
+domain:           test.tld
+mailbox path:     @TMPDIR@/virt
+
+cvm-testclient: Authentication failed, error #100 (Credentials rejected)
+out of scope:     0
+
+cvm-testclient: Authentication failed, error #100 (Credentials rejected)
+out of scope:     1
 END_OF_TEST_RESULTS
 
 
@@ -1059,13 +1102,59 @@ mailbox path:     @TMPDIR@/virt
 END_OF_TEST_RESULTS
 
 
-##### Test tests/command-case #####
+##### Test tests/vmailmgr-upper-domain #####
+
+runtest() {
+$src/cvm-testclient $src/cvm-vmailmgr virt Test.TLD pass
+}
+run_compare_test tests/vmailmgr-upper-domain  <<END_OF_TEST_RESULTS
+user name:        virt
+user ID:          @UID@
+group ID:         @GID@
+real name:        (null)
+directory:        @TMPDIR@/virt
+shell:            (null)
+group name:       (null)
+system user name: user
+system directory: @TMPDIR@
+domain:           test.tld
+mailbox path:     @TMPDIR@/virt
+END_OF_TEST_RESULTS
+
+
+##### Test tests/sasl-login2 #####
+
+runtest() {
+sasltest LOGIN\ cHdmdXNlcg== <<EOF
+dGVzdHBhc3M=
+EOF
+}
+run_compare_test tests/sasl-login2  <<END_OF_TEST_RESULTS
+sasl-auth-test: AUTH LOGIN PLAIN
++ UGFzc3dvcmQ6^M
+sasl-auth-test: SASL AUTH LOGIN username=pwfuser
+END_OF_TEST_RESULTS
+
+
+##### Test tests/vmlookup-baduser #####
+
+runtest() {
+CVM_LOOKUP_SECRET=secret \
+$src/cvm-testclient $tmp/cvm-vmailmgr-lookup missing test.tld
+}
+run_compare_test tests/vmlookup-baduser  <<END_OF_TEST_RESULTS
+cvm-testclient: Authentication failed, error #100 (Credentials rejected)
+out of scope:     0
+END_OF_TEST_RESULTS
+
+
+##### Test tests/pwfile #####
 
 runtest() {
 make_pwfile
-$src/cvm-testclient $src/cvm-pwfile PWFUser '' testpass
+$src/cvm-testclient $src/cvm-pwfile pwfuser '' testpass
 }
-run_compare_test tests/command-case  <<END_OF_TEST_RESULTS
+run_compare_test tests/pwfile  <<END_OF_TEST_RESULTS
 user name:        pwfuser
 user ID:          123
 group ID:         456
@@ -1080,6 +1169,30 @@ mailbox path:     (null)
 END_OF_TEST_RESULTS
 
 
+##### Test tests/qmail-lookup-baddomain #####
+
+runtest() {
+env CVM_LOOKUP_SECRET=secret \
+$src/cvm-testclient $tmp/cvm-qmail-lookup user other.dom
+}
+run_compare_test tests/qmail-lookup-baddomain  <<END_OF_TEST_RESULTS
+cvm-testclient: Authentication failed, error #100 (Credentials rejected)
+out of scope:     1
+END_OF_TEST_RESULTS
+
+
+##### Test tests/vmlookup-baddomain #####
+
+runtest() {
+CVM_LOOKUP_SECRET=secret \
+$src/cvm-testclient $tmp/cvm-vmailmgr-lookup virt other.tld
+}
+run_compare_test tests/vmlookup-baddomain  <<END_OF_TEST_RESULTS
+cvm-testclient: Authentication failed, error #100 (Credentials rejected)
+out of scope:     1
+END_OF_TEST_RESULTS
+
+
 ##### Test tests/start-pwfile-udp #####
 
 runtest() {
@@ -1093,19 +1206,6 @@ Starting.
 END_OF_TEST_RESULTS
 
 
-##### Test tests/pwfile-udp-bad-password #####
-
-runtest() {
-make_pwfile
-$src/cvm-testclient cvm-udp:127.1.2.3:12345 pwfuser '' testpaxx
-tail -n 1 $tmp/log
-}
-run_compare_test tests/pwfile-udp-bad-password  <<END_OF_TEST_RESULTS
-cvm-testclient: Fatal: Authentication failed, error #100 (Credentials rejected)
-- pwfuser
-END_OF_TEST_RESULTS
-
-
 ##### Test tests/pwfile-udp #####
 
 runtest() {
@@ -1129,6 +1229,19 @@ mailbox path:     (null)
 END_OF_TEST_RESULTS
 
 
+##### Test tests/pwfile-udp-bad-password #####
+
+runtest() {
+make_pwfile
+$src/cvm-testclient cvm-udp:127.1.2.3:12345 pwfuser '' testpaxx
+tail -n 1 $tmp/log
+}
+run_compare_test tests/pwfile-udp-bad-password  <<END_OF_TEST_RESULTS
+cvm-testclient: Authentication failed, error #100 (Credentials rejected)
+- pwfuser
+END_OF_TEST_RESULTS
+
+
 ##### Test tests/v1udp #####
 
 runtest() {
@@ -1181,19 +1294,6 @@ Starting.
 END_OF_TEST_RESULTS
 
 
-##### Test tests/pwfile-local-bad-password #####
-
-runtest() {
-make_pwfile
-$src/cvm-testclient cvm-local:$tmp/socket pwfuser '' testpaxx
-tail -n 1 $tmp/log
-}
-run_compare_test tests/pwfile-local-bad-password  <<END_OF_TEST_RESULTS
-cvm-testclient: Fatal: Authentication failed, error #100 (Credentials rejected)
-- pwfuser
-END_OF_TEST_RESULTS
-
-
 ##### Test tests/pwfile-local #####
 
 runtest() {
@@ -1217,6 +1317,19 @@ mailbox path:     (null)
 END_OF_TEST_RESULTS
 
 
+##### Test tests/pwfile-local-bad-password #####
+
+runtest() {
+make_pwfile
+$src/cvm-testclient cvm-local:$tmp/socket pwfuser '' testpaxx
+tail -n 1 $tmp/log
+}
+run_compare_test tests/pwfile-local-bad-password  <<END_OF_TEST_RESULTS
+cvm-testclient: Authentication failed, error #100 (Credentials rejected)
+- pwfuser
+END_OF_TEST_RESULTS
+
+
 ##### Test tests/v1local #####
 
 runtest() {
