@@ -1,5 +1,5 @@
 /* cvm/client.c - CVM client library
- * Copyright (C)2006  Bruce Guenter <bruce@untroubled.org>
+ * Copyright (C)2008  Bruce Guenter <bruce@untroubled.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,15 +25,19 @@
 
 #include <sysdeps.h>
 #include <net/socket.h>
+#include <str/iter.h>
+#include <str/str.h>
 
 #include "v2client.h"
 #include "credentials.h"
 #include "protocol.h"
+#include "random.h"
 
 const char* cvm_client_account_split_chars = "@";
 
-static unsigned char buffer[CVM_BUFSIZE];
-static unsigned buflen;
+static struct cvm_packet request;
+static struct cvm_packet response;
+
 static struct 
 {
   unsigned type;
@@ -41,30 +45,30 @@ static struct
 } offsets[CVM_BUFSIZE/2];
 static str randombytes;
 
-/* Buffer management code ****************************************************/
-static int parse_buffer(void)
+/* Packet management code ****************************************************/
+static int parse_packet(struct cvm_packet* p)
 {
   unsigned i;
   unsigned o;
-  if (buflen < 3)
+  if (p->length < 3)
     return CVME_BAD_MODDATA;
-  if (buffer[1] != randombytes.len)
+  if (p->data[1] != randombytes.len)
     return CVME_BAD_MODDATA;
-  if (memcmp(buffer+2, randombytes.s, randombytes.len) != 0)
+  if (memcmp(p->data+2, randombytes.s, randombytes.len) != 0)
     return CVME_BAD_MODDATA;
-  if (buffer[0] != 0)
-    return buffer[0];
-  if (buffer[buflen-1] != 0)
+  if (p->data[p->length-1] != 0)
     return CVME_BAD_MODDATA;
-  /* This funny loop gives all the strings in the buffer NUL termination. */
-  for (i = 0, o = buffer[1] + 2;
-       o < sizeof buffer && buffer[o] != 0;
-       ++i, o += buffer[o+1] + 2) {
-    offsets[i].type = buffer[o];
+  /* This funny loop gives all the strings in the p->data NUL termination. */
+  for (i = 0, o = p->data[1] + 2;
+       o < sizeof p->data && p->data[o] != 0;
+       ++i, o += p->data[o+1] + 2) {
+    offsets[i].type = p->data[o];
     offsets[i].start = o+2;
-    buffer[o] = 0;
+    p->data[o] = 0;
   }
   offsets[i].type = offsets[i].start = 0;
+  if (p->data[0] != 0)
+    return p->data[0];
   /* Extract required and common facts. */
   if (cvm_client_fact_str(CVM_FACT_USERNAME, &cvm_fact_username, &i) ||
       cvm_client_fact_uint(CVM_FACT_USERID, &cvm_fact_userid) ||
@@ -81,43 +85,74 @@ static int parse_buffer(void)
   return 0;
 }
 
-static unsigned char* buffer_add(unsigned char* ptr, unsigned type,
-				 unsigned len, const char* data)
+static unsigned packet_add(struct cvm_packet* p, unsigned type,
+			   unsigned len, const char* data)
 {
-  if (ptr - buffer + len + 2 >= CVM_BUFSIZE-1) return 0;
+  unsigned char* ptr;
+  if (p->length + len + 2 >= CVM_BUFSIZE-1)
+    return 0;
+  ptr = p->data + p->length;
   *ptr++ = type;
   *ptr++ = len;
   memcpy(ptr, data, len);
-  return ptr + len;
+  p->length += len + 2;
+  return 1;
 }
 
-static unsigned build_buffer(unsigned count,
-			     const struct cvm_credential* credentials)
+static void make_randombytes(void)
+{
+  static int initialized = 0;
+  unsigned i;
+  const char *e;
+
+  if (!initialized) {
+    cvm_random_init();
+
+    if (randombytes.len == 0) {
+      if ((e = getenv("CVM_RANDOM_BYTES")) != 0)
+	i = atoi(e);
+      else
+	i = 8;
+      str_ready(&randombytes, i);
+      randombytes.len = i;
+    }
+    initialized = 1;
+  }
+
+  cvm_random_fill((unsigned char*)randombytes.s, randombytes.len);
+}
+
+static unsigned build_packet(struct cvm_packet* p,
+			     unsigned count,
+			     const struct cvm_credential* credentials,
+			     int addrandom)
 {
   const char* env;
-  unsigned char* ptr;
   unsigned i;
   int has_secret;
 
-  /* FIXME: generate real random data here */
-  ptr = buffer_add(buffer, CVM2_PROTOCOL, randombytes.len, randombytes.s);
+  if (addrandom)
+    make_randombytes();
+  else
+    randombytes.len = 0;
+  p->length = 0;
+  if (!packet_add(p, CVM2_PROTOCOL, randombytes.len, randombytes.s))
+    return 0;
 
   for (i = 0, has_secret = 0; i < count; ++i, ++credentials) {
     if (credentials->type == CVM_CRED_SECRET)
       has_secret = 1;
-    if ((ptr = buffer_add(ptr, credentials->type,
-			  credentials->value.len,
-			  credentials->value.s)) == 0)
+    if (!packet_add(p, credentials->type,
+		    credentials->value.len, credentials->value.s))
       return 0;
   }
 
   if (!has_secret
       && (env = getenv("CVM_LOOKUP_SECRET")) != 0)
-    if ((ptr = buffer_add(ptr, CVM_CRED_SECRET, strlen(env), env)) == 0)
+    if (!packet_add(p, CVM_CRED_SECRET, strlen(env), env))
       return 0;
 
-  *ptr++ = 0;
-  buflen = ptr - buffer;
+  p->data[p->length++] = 0;
   return 1;
 }
 
@@ -135,7 +170,7 @@ int cvm_client_fact_str(unsigned number, const char** data, unsigned* length)
   
   while (offsets[o].type != 0) {
     if (offsets[o++].type == number) {
-      *length = (*data = buffer + offsets[o-1].start)[-1];
+      *length = (*data = response.data + offsets[o-1].start)[-1];
       err = 0;
       break;
     }
@@ -188,24 +223,62 @@ int cvm_client_split_account(str* account, str* domain)
 }
 
 /* Top-level wrapper *********************************************************/
-int cvm_client_authenticate(const char* module, unsigned count,
+int cvm_client_authenticate(const char* modules, unsigned count,
 			    const struct cvm_credential* credentials)
 {
   int result;
   void (*oldsig)(int);
-  if (!build_buffer(count, credentials))
+  int addrandom;
+  static str module_list;
+  striter i;
+  unsigned long u;
+
+  /* Make a copy of the module list so we can make the strings NUL
+   * terminated internally. */
+  if (!str_copys(&module_list, modules))
+    return CVME_IO | CVME_FATAL;
+  str_subst(&module_list, ',', '\0');
+
+  /* Set addrandom to true if any module uses UDP. */
+  addrandom = 0;
+  striter_loop(&i, &module_list, '\0') {
+    if (memcmp(i.startptr, "cvm-udp:", 8) == 0) {
+      addrandom = 1;
+      break;
+    }
+  }
+
+  if (!build_packet(&request, count, credentials, addrandom))
     return CVME_GENERAL;
   
   oldsig = signal(SIGPIPE, SIG_IGN);
-  if (!memcmp(module, "cvm-udp:", 8))
-    result = cvm_xfer_udp(module+8, buffer, &buflen);
-  else if (!memcmp(module, "cvm-local:", 10))
-    result = cvm_xfer_local(module+10, buffer, &buflen);
-  else {
-    if (!memcmp(module, "cvm-command:", 12)) module += 12;
-    result = cvm_xfer_command(module, buffer, &buflen);
+
+  /* Invoke each module in the list, exiting when any module produces a
+   * non-PERMFAIL result, or when it produces a PERMFAIL result with
+   * OUTOFSCOPE set to 0. */
+  striter_loop(&i, &module_list, '\0') {
+    if (!memcmp(i.startptr, "cvm-udp:", 8))
+      result = cvm_xfer_udp_packets(i.startptr+8, &request, &response);
+    else if (!memcmp(i.startptr, "cvm-local:", 10))
+      result = cvm_xfer_local_packets(i.startptr+10, &request, &response);
+    else {
+      if (!memcmp(i.startptr, "cvm-command:", 12))
+	i.startptr += 12;
+      result = cvm_xfer_command_packets(i.startptr, &request, &response);
+    }
+    /* Note: the result returned by cvm_xfer_* indicates if transmission
+     * succeeded, not the actual result of validation.  The validation
+     * result is returned by parse_packet. */
+    if (result == 0)
+      result = parse_packet(&response);
+    /* Return success and temporary failures. */
+    if (result != CVME_PERMFAIL)
+      break;
+    /* Also return permanent failure if the result is in scope. */
+    if (cvm_client_fact_uint(CVM_FACT_OUTOFSCOPE, &u) == 0
+	&& u == 0)
+      break;
   }
   signal(SIGPIPE, oldsig);
-  if (result != 0) return result;
-  return parse_buffer();
+  return result;
 }
